@@ -36,10 +36,6 @@
 #include <libfreebob/freebob.h>
 #include <libfreebob/freebob_streaming.h>
 
-#include <jack/driver.h>
-#include <jack/engine.h>
-#include <jack/types.h>
-
 #include <string.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -52,15 +48,14 @@
 #include <pthread.h>
 #include <semaphore.h>
 
-#include <jack/internal.h>
-#include <jack/types.h>
-#include <jack/ringbuffer.h>
 #include <jack/driver.h>
 #include <jack/engine.h>
 #include <jack/types.h>
-#include <jack/thread.h>
 
+#ifdef FREEBOB_DRIVER_WITH_MIDI
+#include <jack/thread.h>
 #include <alsa/asoundlib.h>
+#endif
 
 // debug print control flags
 #define DEBUG_LEVEL_BUFFERS           	(1<<0)
@@ -145,16 +140,19 @@ struct _freebob_jack_settings {
 	int buffer_size_set;
 	jack_nframes_t buffer_size;
 
-        int port_set;
-        int port;
-       
-        int node_id_set;
-        int node_id;
+	int port_set;
+	int port;
+	
+	int node_id_set;
+	int node_id;
 
 	int playback_ports;
 	int capture_ports;
+	
+	jack_nframes_t capture_frame_latency;
+	jack_nframes_t playback_frame_latency;
 
-        freebob_handle_t fb_handle;
+	freebob_handle_t fb_handle;
 };
 
 #ifdef FREEBOB_DRIVER_WITH_MIDI
@@ -226,7 +224,10 @@ struct _freebob_driver
     JSList                       *monitor_ports;
     channel_t                     playback_nchannels;
     channel_t                     capture_nchannels;
-    	
+
+	jack_nframes_t  playback_frame_latency;
+	jack_nframes_t  capture_frame_latency;
+
 	freebob_device_info_t device_info;
 	freebob_options_t device_options;
 
