@@ -19,7 +19,7 @@
     * 2002/08/23 - modify for libsndfile 1.0.0 <andy@alsaplayer.org>
     * 2003/05/26 - use ringbuffers - joq
     
-    $Id: capture_client.c 676 2004-03-27 22:08:55Z joq $
+    $Id: capture_client.c 994 2006-11-08 18:28:15Z joq $
 */
 
 #include <stdio.h>
@@ -207,9 +207,6 @@ run_disk_thread (jack_thread_info_t *info)
 			 PRIu32 ".\n", info->rb_size);
 		info->status = EPIPE;
 	}
-	if (info->status) {
-		unlink (info->path);
-	}
 }
 
 void
