@@ -15,7 +15,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: driver.h,v 1.20 2003/11/24 23:18:56 node Exp $
+    $Id: driver.h 945 2006-05-04 15:14:45Z pbd $
 */
 
 #ifndef __jack_driver_h__
@@ -27,7 +27,7 @@
 #include <jack/driver_interface.h>
 
 typedef float         gain_t;
-typedef long          channel_t;
+typedef unsigned long channel_t;
 
 typedef	enum  {
 	Lock = 0x1,
