@@ -21,7 +21,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: internal.h,v 1.82 2005/08/22 20:48:24 rncbc Exp $
+    $Id: internal.h 948 2006-05-15 17:12:44Z pbd $
 */
 
 #ifndef __jack_internal_h__
@@ -42,6 +42,17 @@ extern void jack_error (const char *fmt, ...);
 #include <jack/types.h>
 #include <jack/port.h>
 #include <jack/transport.h>
+
+typedef enum {
+	JACK_TIMER_SYSTEM_CLOCK,
+	JACK_TIMER_CYCLE_COUNTER,
+	JACK_TIMER_HPET,
+} jack_timer_type_t;
+
+void        jack_init_time ();
+void        jack_set_clock_source (jack_timer_type_t);
+const char* jack_clock_source_name (jack_timer_type_t);
+
 #include <sysdeps/time.h>
 #include <sysdeps/atomicity.h>
 
@@ -130,6 +141,7 @@ typedef struct {
     jack_time_t           sync_time_left;
     jack_frame_timer_t    frame_timer;
     int32_t		  internal;
+    jack_timer_type_t     clock_source;
     pid_t                 engine_pid;
     jack_nframes_t	  buffer_size;
     int8_t		  real_time;
@@ -418,7 +430,7 @@ extern char *jack_tmpdir;
 
 extern char *jack_user_dir (void);
 
-extern char *jack_server_dir (const char *server_name);
+extern char *jack_server_dir (const char *server_name, char *server_dir);
 
 extern void *jack_zero_filled_buffer;
 
