@@ -16,7 +16,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: jackd.c 978 2006-06-20 22:15:52Z joq $
+    $Id: jackd.c 1010 2007-02-23 19:23:07Z paul $
 */
 
 #include <config.h>
@@ -507,12 +507,13 @@ main (int argc, char *argv[])
 
 {
 	jack_driver_desc_t * desc;
-	const char *options = "-ad:P:uvshVRTFl:t:mn:p:c:";
+	const char *options = "-ad:P:uvshVRTFlt:mn:p:c:";
 	struct option long_options[] = 
 	{ 
 		{ "driver", 1, 0, 'd' },
 		{ "verbose", 0, 0, 'v' },
 		{ "help", 0, 0, 'h' },
+		{ "tmpdir-location", 0, 0, 'l' },
 		{ "port-max", 1, 0, 'p' },
 		{ "no-mlock", 0, 0, 'm' },
 		{ "name", 1, 0, 'n' },
@@ -569,6 +570,11 @@ main (int argc, char *argv[])
 			frame_time_offset = JACK_MAX_FRAMES - atoi(optarg); 
 			break;
 
+		case 'l':
+			/* special flag to allow libjack to determine jackd's idea of where tmpdir is */
+			printf ("%s\n", jack_tmpdir);
+			exit (0);
+
 		case 'm':
 			do_mlock = 0;
 			break;
