@@ -17,7 +17,7 @@
     along with this program; if not, write to the Free Software 
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: client.c 948 2006-05-15 17:12:44Z pbd $
+    $Id: client.c 1010 2007-02-23 19:23:07Z paul $
 */
 
 #include <config.h>
@@ -27,6 +27,7 @@
 #include <fcntl.h>
 #include <stdarg.h>
 #include <stdio.h>
+#include <limits.h>
 #ifdef HAVE_STDINT_H
 #include <stdint.h>
 #endif
@@ -67,7 +68,6 @@
 
 static pthread_mutex_t client_lock;
 static pthread_cond_t  client_ready;
-void *jack_zero_filled_buffer = NULL;
 #ifdef ARCH_X86
 int cpu_type = 0;
 #endif /* ARCH_X86 */
@@ -206,6 +206,42 @@ init_cpu ()
 
 #endif /* USE_DYNSIMD */
 
+char *jack_tmpdir = DEFAULT_TMP_DIR;
+
+static int
+jack_get_tmpdir ()
+{
+	FILE* in;
+	size_t len;
+	char buf[PATH_MAX+2]; /* allow tmpdir to live anywhere, plus newline, plus null */
+
+	if ((in = popen ("jackd -l", "r")) == NULL) {
+		return -1;
+	}
+
+	if (fgets (buf, sizeof (buf), in) == NULL) {
+		fclose (in);
+		return -1;
+	}
+
+	len = strlen (buf);
+
+	if (buf[len-1] != '\n') {
+		/* didn't get a whole line */
+		fclose (in);
+		return -1;
+	}
+
+	jack_tmpdir = (char *) malloc (len);
+	memcpy (jack_tmpdir, buf, len-1);
+	jack_tmpdir[len-1] = '\0';
+	
+	fprintf (stderr, "JACK tmpdir identified as [%s]\n", jack_tmpdir);
+
+	fclose (in);
+	return 0;
+}
+
 void 
 jack_error (const char *fmt, ...)
 {
@@ -622,6 +658,10 @@ _start_server (const char *server_name)
 	snprintf(filename, 255, "%s/.jackdrc", getenv("HOME"));
 	fp = fopen(filename, "r");
 
+	if (!fp) {
+		fp = fopen("/etc/jackdrc", "r");
+	}
+	/* if still not found, check old config name for backwards compatability */
 	if (!fp) {
 		fp = fopen("/etc/jackd.conf", "r");
 	}
@@ -691,7 +731,7 @@ _start_server (const char *server_name)
 
 	/* If execv() succeeds, it does not return.  There's no point
 	 * in calling jack_error() here in the child process. */
-	perror ("exec of JACK server failed");
+	fprintf (stderr, "exec of JACK server (command = \"%s\") failed: %s\n", command, strerror (errno));
 }
 
 int
@@ -906,17 +946,6 @@ jack_attach_port_segment (jack_client_t *client, jack_port_type_id_t ptid)
 		return -1;
 	}
 
-	/* The first chunk of the audio port segment will be set by
-	 * the engine to be a zero-filled buffer.  This hasn't been
-	 * done yet, but it will happen before the process cycle
-	 * (re)starts. 
-	 */
-
-	if (ptid == JACK_AUDIO_PORT_TYPE) {
-		jack_zero_filled_buffer =
-			jack_shm_addr (&client->port_segment[ptid]);
-	}
-
 	return 0;
 }
 
@@ -951,6 +980,14 @@ jack_client_open (const char *client_name,
 	jack_varargs_parse (options, ap, &va);
 	va_end (ap);
 
+	/* External clients need to know where the tmpdir used for
+	   communication with the server lives
+	*/
+	if (jack_get_tmpdir ()) {
+		*status |= JackFailure;
+		return NULL;
+	}
+
 	/* External clients need this initialized.  It is already set
 	 * up in the server's address space for internal clients.
 	 */
@@ -1117,8 +1154,6 @@ jack_default_server_name (void)
 	return server_name;
 }
 
-char *jack_tmpdir = DEFAULT_TMP_DIR;
-
 /* returns the name of the per-user subdirectory of jack_tmpdir */
 char *
 jack_user_dir (void)
@@ -2240,3 +2275,4 @@ jack_port_type_size(void)
 {
 	return JACK_PORT_TYPE_SIZE;
 }
+
