@@ -18,7 +18,7 @@
   Thread creation function including workarounds for real-time scheduling
   behaviour on different glibc versions.
 
-  $Id: thread.c,v 1.9 2005/04/15 21:27:15 pbd Exp $
+  $Id: thread.c 948 2006-05-15 17:12:44Z pbd $
 
 */
 
@@ -159,6 +159,12 @@ jack_client_create_thread (jack_client_t* client,
 		return result;
 	}
 
+        result = pthread_attr_setstacksize(&attr, THREAD_STACK); 
+        if (result) {
+                log_result("setting thread stack size", result);
+                return result;
+        }
+
 	thread_args = (jack_thread_arg_t *) malloc (sizeof (jack_thread_arg_t));
 
 	thread_args->client = client;
